import os
import click
from griddot.provision_keycloak import import_realm, get_user_uuid_by_username, get_access_token, delete_user
from griddot.tools import encrypt_secrets, create_new_key, decrypt_secrets, helm_template
from griddot.tools import helm_list_charts


@click.group()
def cli():
    """CLI for GridDot platform tools"""
    pass


@click.group()
def keycloak():
    """Tools for managing keycloak"""
    pass


@click.group()
def secrets():
    """Tools for managing secrets"""
    pass


@click.group()
def helm():
    """Tools for managing deployments"""
    pass


cli.add_command(secrets)
cli.add_command(keycloak)
cli.add_command(helm)


@keycloak.command("provision", short_help="Provision Keycloak with realms and users")
@click.option('--url', help='Keycloak server URL')
@click.option('--username', help='User name')
@click.option('--password', help='Username password')
@click.option('--realms-dir', help='Path to the directory with realm JSON files')
@click.option('--delete-user-when-provisioned', type=bool, default=True)
@click.option('--email-password', help='Email provider password')
def keycloak_provision(url, username, password, realms_dir, delete_user_when_provisioned, email_password):
    """Keycloak provisioning tool"""

    for realm_path in os.listdir(realms_dir):
        full_path = os.path.join(realms_dir, realm_path)
        print(f"Importing realm from {full_path}")
        import_realm(url, username, password, full_path, email_password, True)

    if delete_user_when_provisioned.lower() == 'yes':
        token = get_access_token(url, username, password)
        user_id = get_user_uuid_by_username(url, "master", token, username)
        delete_user(url, "master", token, user_id)
        print(f"Deleted user {username} from master realm.")


@secrets.command("decrypt", short_help="Decrypt secrets files using the private key .secrets/key.pem")
@click.option('--file', '-f', multiple=True, help='Path to the encrypted secrets file(s)')
def secrets_decrypt(file: tuple[str]):
    decrypt_secrets(list(file))


@secrets.command("create", short_help="Create a new RSA key pair: .secrets/key.pem and .secrets/key.pub")
def secrets_create():
    create_new_key()


@secrets.command("encrypt", short_help="Encrypt secrets files using the public key .secrets/key.pub")
@click.option('--file', '-f', multiple=True, help='Path to the secrets file(s) to encrypt')
def secrets_encrypt(file: tuple[str]):
    encrypt_secrets(list(file))


@helm.command("list", short_help="List all deployments from helm repository")
def deploy_list():
    helm_charts = helm_list_charts()
    print("Possible deployments:")
    for chart in helm_charts:
        print(f"- {chart}")


@helm.command("template",
              short_help="Templates a Kubernetes deployment using helm for podman kube play command",
              help="List deployments with `griddot helm list`")
@click.argument("deployment")
@click.option('--values', '-f', multiple=True, help='Path to the values file for the helm chart')
def deploy_list(deployment: str, values: tuple[str]):
    if deployment not in helm_list_charts():
        raise click.BadParameter(f"Deployment '{deployment}' not found in helm repository. "
                                 f"Use `griddot helm list` to see available deployments.")

    for value in values:
        if not os.path.exists(value):
            raise click.BadParameter(f"Values file '{value}' does not exist.")

    templated_yaml = helm_template(deployment, values)
    print(templated_yaml)



import subprocess
import requests
from requests.auth import HTTPBasicAuth
import yaml
from griddot.tools.utils import get_container_command


def helm_list_charts():
    encoded_project = requests.utils.quote("griddot/packages", safe="")
    url = f"https://gitlab.com/api/v4/projects/{encoded_project}/packages/helm/stable/index.yaml"

    # GitLab expects: username = anything (often 'gitlab-ci-token'), password = the token
    auth = HTTPBasicAuth("helm-user", "glpat-mKckaB2sg2vC74xzFWWB")  # or "gitlab-ci-token", token

    response = requests.get(url, auth=auth)
    response.raise_for_status()

    index = yaml.safe_load(response.text)
    charts = list(index.get("entries", {}).keys())
    charts = [f"griddot/{chart}" for chart in charts]
    return charts


def helm_template(deployment: str, values: tuple[str] = None):
    """
    Create a Kubernetes deployment using helm, from the possible deployments in list_helm_charts.
    """
    container_cmd = get_container_command()
    if len(values) > 0:
        values_mount = " ".join([f"-v ./{value}:/{value.split('/')[-1]}" for value in values])
        values_option = "".join([f"-f /{value.split('/')[-1]} " for value in values])
    else:
        values_option = ""
        values_mount = ""

    result = subprocess.run(
        f"{container_cmd} run --rm {values_mount} registry.gitlab.com/griddot/packages/helm:latest helm template {deployment} {values_option}",
        shell=True, check=True, capture_output=True
    )

    return result.stdout.decode('utf-8')



import os

from click.testing import CliRunner
from griddot.cli import cli

from python.griddot.tools import rsa_generate, rsa_encrypt, rsa_decrypt


def test_encryption_decryption():
    private_key, public_key = rsa_generate()

    original_text = "Hello, GridDot!"
    encrypted_text = rsa_encrypt(public_key, original_text)
    assert encrypted_text != original_text  # Ensure encryption changed the text

    decrypted_text = rsa_decrypt(private_key, encrypted_text)
    assert decrypted_text == original_text  # Ensure decryption returns the original text


def test_cli_encrypt():
    runner = CliRunner()
    result = runner.invoke(cli, [
        'encrypt-secrets',
    ])

    assert result.exit_code == 0, result.output


def test_cli():
    runner = CliRunner()

    # Generate keys
    result = runner.invoke(cli, [
        'create-key'
    ])

    assert result.exit_code == 0, result.output

    # Test encryption
    with open('file1.txt', 'w') as f:
        f.write('This is a secret file 1.')

    with open('file2.txt', 'w') as f:
        f.write('This is a secret file 2.')

    file1_abs_path = os.path.abspath('file1.txt')
    file2_abs_path = os.path.abspath('file2.txt')

    result = runner.invoke(cli, [
        'encrypt-secrets',
        '-f', file1_abs_path,
        '--file', file2_abs_path
    ])

    assert result.exit_code == 0, result.output

    # Test decryption
    result = runner.invoke(cli, [
        'decrypt-secrets',
        '-f', file1_abs_path + '.encrypted',
        '--file', file2_abs_path + '.encrypted'
    ])

    assert result.exit_code == 0, result.output

    with open('file1.txt.encrypted', 'r') as f:
        encrypted_content1 = f.read()

    with open('file2.txt.encrypted', 'r') as f:
        encrypted_content2 = f.read()

    assert encrypted_content1 != 'This is a secret file 1.'
    assert encrypted_content2 != 'This is a secret file 2.'

    # Test decryption without specifying files
    os.remove('file1.txt')
    os.remove('file2.txt')

    result = runner.invoke(cli, [
        'decrypt-secrets'
    ])

    assert result.exit_code == 0, result.output

    with open('file1.txt', 'r') as f:
        decrypted_content1 = f.read()
    with open('file2.txt', 'r') as f:
        decrypted_content2 = f.read()

    assert decrypted_content1 == 'This is a secret file 1.'
    assert decrypted_content2 == 'This is a secret file 2.'

    # Test encryption without specifying files
    with open('file1.txt.encrypted', 'w') as f:
        f.write('')
    with open('file2.txt.encrypted', 'w') as f:
        f.write('')

    result = runner.invoke(cli, [
        'encrypt-secrets'
    ])

    assert result.exit_code == 0, result.output

    result = runner.invoke(cli, [
        'decrypt-secrets'
    ])

    assert result.exit_code == 0, result.output

    with open('file1.txt', 'w') as f:
        f.write('This is a secret file 1.')
    with open('file2.txt', 'w') as f:
        f.write('This is a secret file 2.')

    os.remove('file1.txt')
    os.remove('file2.txt')
    os.remove('file1.txt.encrypted')
    os.remove('file2.txt.encrypted')



import subprocess
import pytest
from click.testing import CliRunner

from griddot.cli import cli
from griddot.provision_keycloak import get_access_token, wait_for_keycloak_to_start, import_realm, get_realms, \
    import_users, get_client_roles, get_user_uuid_by_username, get_client_roles_for_user, import_client_roles, \
    import_client, create_realm

USERNAME = "temp-admin"
PASSWORD = "temp-admin"
EMAIL_PASSWORD = "ppgb ddgj sjki xbya"


@pytest.fixture(scope="module", autouse=True)
def setup_once_for_module():
    # This runs once before any tests in this module
    setup_keycloak(False, False)
    yield
    # This runs once after all tests in this module


def setup_keycloak(clean, build_images):
    if build_images:
        build_keycloak_images()

    if clean:
        run_keycloak()


def run_keycloak():
    cwd = "../../containers/keycloak"
    print("Deleting existing Keycloak resources...")
    subprocess.run(f"podman kube play --down --force dev.yaml", check=False, shell=True, cwd=cwd)
    print("Starting Keycloak...")
    subprocess.run(f"podman kube play --replace dev.yaml", check=True, shell=True, cwd=cwd)


def build_keycloak_images():
    print("Building Keycloak images...")
    subprocess.run("python3 build_images.py", check=True, shell=True, cwd="../../containers")


def wait_for_keycloak_and_get_token(url="https://localhost:8443"):
    wait_for_keycloak_to_start(url)
    token = get_access_token(url, USERNAME, PASSWORD)
    return url, token


def test_import_realm():
    url = "https://localhost:8443"
    master_realm = "../../containers/keycloak/realms/master-realm.json"
    import_realm(url, USERNAME, PASSWORD, master_realm, EMAIL_PASSWORD, False)

    platform_realm = "../../containers/keycloak/realms/platform-realm.json"
    import_realm(url, USERNAME, PASSWORD, platform_realm, EMAIL_PASSWORD, False)
    import_realm(url, USERNAME, PASSWORD, platform_realm, EMAIL_PASSWORD, False)  # idempotency test

    token = get_access_token(url, USERNAME, PASSWORD)
    realms = get_realms(url, token)
    realms_names = [realm["realm"] for realm in realms]
    assert "master" in realms_names, "Master realm not found"
    assert "platform" in realms_names, "Platform realm not found"


def test_cli():
    url, _ = wait_for_keycloak_and_get_token()

    runner = CliRunner()
    result = runner.invoke(cli, [
        'provision-keycloak',
        '--url', "https://localhost:8443",
        '--username', USERNAME,
        '--password', PASSWORD,
        '--realms-dir', '../../containers/keycloak/realms',
        '--delete-user-when-provisioned', 'Yes',
        '--email-password', EMAIL_PASSWORD
    ])

    assert result.exit_code == 0, result.output


def test_creating_realm():
    url, token = wait_for_keycloak_and_get_token()
    realm = {
        "realm": "test-realm",
        "enabled": True,
    }
    create_realm(url, realm, token)

    all_realms = get_realms(url, token)
    assert any(realm["realm"] == "test-realm" for realm in all_realms), "test-realm should be created"

    # ensure creating the realm is idempotent
    create_realm(url, realm, token)


def configure_email_provider(url, realm_name, token, email_config):
    pass


def test_import_users():
    url, token = wait_for_keycloak_and_get_token()

    realm_name = "import-users-realm"
    realm = {
        "realm": realm_name,
        "enabled": True,
    }

    create_realm(url, realm, token)
    create_realm(url, realm, token)

    client = {
        "clientId": "my-client",
        "name": "My Admin Client",
        "enabled": True,
        "protocol": "openid-connect",
    }

    client_uuid = import_client(url, realm_name, token, client)
    client_uuid2 = import_client(url, realm_name, token, client)  # idempotency check
    assert client_uuid == client_uuid2, "Client ID should be the same on idempotent import"

    roles = [
        {
            "name": "my-admin",
            "description": "My admin role",
            "clientRole": True,
        },
        {
            "name": "myy-user",
            "description": "My user role",
            "clientRole": True,
        },
    ]
    import_client_roles(url, realm_name, token, client_uuid, roles)
    import_client_roles(url, realm_name, token, client_uuid, roles)  # idempotency check

    users_to_import = [
        {
            "username": "ali",
            "email": "ali@example.com",
            "enabled": True,
            "credentials": [{"type": "password", "value": "test"}],
            "clientRoles": {
                "my-client": ["my-admin"],
            }
        },
        {
            "username": "bob",
            "email": "bob@example.com",
            "enabled": True,
            "credentials": [{"type": "password", "value": "test"}],
            "clientRoles": {
                "my-client": ["myy-user"],
            },
        },
    ]

    import_users(url, realm_name, token, users_to_import)
    import_users(url, realm_name, token, users_to_import)  # idempotency check

    client_roles = {role["name"] for role in get_client_roles(url, realm_name, token, client_uuid)}
    assert "my-admin" in client_roles, "my-admin role should have been created"
    assert "myy-user" in client_roles, "myy-user role should have been created"

    for user_spec in users_to_import:
        uuid = get_user_uuid_by_username(url, realm_name, token, user_spec["username"])
        user_roles = get_client_roles_for_user(url, realm_name, token, uuid, client_uuid)
        user_roles_names = {role["name"] for role in user_roles}
        for role in user_spec["clientRoles"]["my-client"]:
            assert role in user_roles_names, f"Role {role} should be assigned to user {user_spec['username']}"



import json
import sys
import time
import requests
import urllib3

# Disable warnings for self-signed certs
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

ADMIN_USERNAME = "temp-admin"
DEFAULT_URL = "https://localhost:8443"


def import_realm(
        url: str,
        admin_username: str,
        admin_password: str,
        realm_json_path: str,
        email_provider_password: str | None,
        send_emails: bool
):
    print(f'Provisioning Keycloak at {url}')
    wait_for_keycloak_to_start(url)
    token = get_access_token(url, admin_username, admin_password)

    realm = json.load(open(realm_json_path, "r", encoding="utf-8"))

    if email_provider_password:
        if "smtpServer" in realm:
            realm["smtpServer"]["password"] = email_provider_password

    create_realm(url, realm, token)
    send_emails_to_users(url, realm["realm"], token, realm.get("users", []), send_emails=send_emails)


def get_user_uuid_by_username(url, realm, token, username):
    all_users = get_users(url, realm, token)
    for user in all_users:
        if user["username"] == username:
            return user["id"]

    raise Exception(f"User {username} not found in realm {realm}")


def wait_for_keycloak_to_start(url):
    max_retries = 60
    retries = max_retries
    while retries > 0:
        try:
            response = requests.get(url, verify=False)
            print()
            if response.status_code == 200:
                print("Keycloak is up and running.", flush=True)
                break
            else:
                print(f"Keycloak is not ready yet. Status code: {response.status_code}")
        except requests.exceptions.RequestException:
            if retries == max_retries:
                print(f"Keycloak is not ready yet .", end="", flush=True)
            else:
                print(".", end="", flush=True)
        time.sleep(1)
        retries -= 1
    else:
        print("Keycloak did not start in time.", flush=True)
        sys.exit(1)


def get_access_token(url, username, password):
    token_url = f"{url}/realms/master/protocol/openid-connect/token"
    data = {
        "client_id": "admin-cli",
        "username": username,
        "password": password,
        "grant_type": "password"
    }
    response = requests.post(token_url, data=data, verify=False)
    if response.status_code != 200:
        print("Failed to get access token:", response.text)
        sys.exit(1)
    return response.json()["access_token"]


def send_emails_to_users(url, realm, token, users, send_emails):
    for user in users:
        user_uuid = get_user_uuid_by_username(url, realm, token, user["username"])
        credentials_url = f"{url}/admin/realms/{realm}/users/{user_uuid}/credentials"
        credentials_response = requests.get(credentials_url, headers=get_token_header(token), verify=False)

        has_user_password = False
        if credentials_response.status_code == 200:
            has_user_password = len(credentials_response.json()) > 0

        if not has_user_password:
            if send_emails:
                print(f"Sent email to {user['username']} for password setup.")
                send_email_for_password_setup(url, realm, token, user_uuid)
            else:
                print(f"Dry run: Would send email to {user['username']} for password setup.")


def delete_user(url, realm, token, user_id):
    delete_url = f"{url}/admin/realms/{realm}/users/{user_id}"
    response = requests.delete(
        delete_url,
        headers=get_token_header(token),
        verify=False
    )
    response.raise_for_status()


def get_users(url, realm, token):
    users_url = f"{url}/admin/realms/{realm}/users"
    response = requests.get(users_url, headers=get_token_header(token), verify=False)
    response.raise_for_status()
    return response.json()


def create_realm(url, realm, token):
    realm_name = realm.get("realm")
    if not realm_name:
        raise Exception("The 'realm' dictionary must contain a 'realm' key.")

    th = get_token_header(token)

    # Check if the realm already exists
    check_url = f"{url}/admin/realms/{realm_name}"
    response = requests.get(check_url, headers=th, verify=False)

    # Update realm if it already exists
    if response.status_code == 200:
        update_url = f"{url}/admin/realms/{realm_name}"
        update_response = requests.put(update_url, headers=th, data=json.dumps(realm), verify=False)
        update_response.raise_for_status()

        # When updating a realm, users, roles, and clients are not updated automatically
        if "users" in realm:
            import_users(url, realm_name, token, realm["users"])
        if "roles" in realm:
            if "realmRoles" in realm["roles"]:
                raise Exception("Realm roles are not supported yet ...")
            if "clientRoles" in realm["roles"]:
                for client_id, roles in realm["roles"]["clientRoles"].items():
                    client_uuid = get_client_uuid_by_client_id(url, realm_name, token, client_id)
                    import_client_roles(url, realm_name, token, client_uuid, roles)
        if "clients" in realm:
            for client in realm["clients"]:
                import_client(url, realm_name, token, client)

        print(f"Realm {realm_name} updated successfully.")
        return

    realm_url = f"{url}/admin/realms"
    response = requests.post(realm_url, headers=th, data=json.dumps(realm), verify=False)
    response.raise_for_status()
    print(f"Realm {realm_name} created successfully.")


def send_email_for_password_setup(url, realm, token, user_id):
    send_email_url = f"{url}/admin/realms/{realm}/users/{user_id}/execute-actions-email"
    send_email_response = requests.put(send_email_url,
                                       headers=get_token_header(token),
                                       params={
                                           "lifespan": 3600 * 24 * 7,
                                           "client_id": "account-console",
                                       },
                                       data=json.dumps(["UPDATE_PASSWORD"]),
                                       verify=False)
    send_email_response.raise_for_status()


def get_token_header(token):
    return {"Authorization": f"Bearer {token}",
            "Content-Type": "application/json"}


def get_realms(url, token):
    realms_url = f"{url}/admin/realms"
    response = requests.get(realms_url, headers=get_token_header(token), verify=False)
    response.raise_for_status()
    return response.json()


def import_users(url, realm, token, users):
    users_url = f"{url}/admin/realms/{realm}/users"
    credentials_url = lambda username: f"{url}/admin/realms/{realm}/users/{username}/credentials"
    th = get_token_header(token)

    for user in users:
        # Check if the user already exists
        response = requests.get(users_url, headers=th, params={"username": user["username"]}, verify=False)
        payload = response.json()
        user_uuid = payload[0]["id"] if len(payload) > 0 else None

        # Update user if it already exists
        if response.status_code == 200 and user_uuid:
            update_url = f"{users_url}/{user_uuid}"
            update_response = requests.put(update_url, headers=th, data=json.dumps(user), verify=False)
            update_response.raise_for_status()
            print(f"User {user['username']} updated successfully.")

        # Create a new user if it does not exist
        else:
            create_response = requests.post(users_url, headers=th, data=json.dumps(user), verify=False)
            create_response.raise_for_status()
            user_uuid = create_response.headers.get("Location").split("/")[-1]
            print(f"User {user['username']} created successfully.")

        credentials_response = requests.get(credentials_url(user_uuid), headers=th, verify=False)
        has_user_password = len(credentials_response.json()) > 0
        if not has_user_password:
            send_email_for_password_setup(url, realm, token, user_uuid)
        else:
            print(f"User {user['username']} already has a password. Skipping sending an email.")

        # Assign client roles
        if "clientRoles" in user:
            for client_id, roles_names in user["clientRoles"].items():
                client_uuid = get_client_uuid_by_client_id(url, realm, token, client_id)

                roles = []
                for role_name in roles_names:
                    role = get_client_role_by_role_name(url, realm, token, client_uuid, role_name)
                    roles.append(role)

                assign_roles_url = f"{url}/admin/realms/{realm}/users/{user_uuid}/role-mappings/clients/{client_uuid}"
                assign_response = requests.post(assign_roles_url, headers=th, data=json.dumps(roles), verify=False)
                assign_response.raise_for_status()
                print(f"Assigned roles {roles_names} to user {user['username']} for client {client_id}.")


def import_client(url, realm, token, client) -> str:
    """client: https://www.keycloak.org/docs-api/latest/rest-api/index.html?utm_source=chatgpt.com#ClientRepresentation"""
    clients_url = f"{url}/admin/realms/{realm}/clients"
    th = get_token_header(token)

    # Check if the client already exists
    response = requests.get(clients_url,
                            headers=th,
                            params={"clientId": client["clientId"]},
                            verify=False)

    # Update client if it already exists
    if response.status_code == 200 and len(response.json()) > 0:
        client_uuid = response.json()[0]["id"]
        update_url = f"{clients_url}/{client_uuid}"
        update_response = requests.put(update_url, headers=th, data=json.dumps(client), verify=False)
        update_response.raise_for_status()
        print(f"Client {client['clientId']} ({client_uuid}) updated successfully.")
        return client_uuid

    # Create a new client if it does not exist
    response = requests.post(clients_url, headers=th, data=json.dumps(client), verify=False)
    response.raise_for_status()
    location = response.headers["Location"]
    client_uuid = location.split("/")[-1]
    print(f"Client {client['clientId']} ({client_uuid}) created successfully.")
    return client_uuid


def import_client_roles(url, realm, token, client_uuid, roles):
    """role: https://www.keycloak.org/docs-api/latest/rest-api/index.html?utm_source=chatgpt.com#RoleRepresentation"""
    roles_url = f"{url}/admin/realms/{realm}/clients/{client_uuid}/roles"
    th = get_token_header(token)
    for role in roles:
        role_name = role.get("name")
        if not role_name:
            raise Exception(f"Role name not found in {role}")

        # Check if the role already exists
        response = requests.get(roles_url, headers=th, params={"search": role_name}, verify=False)

        # Update role if it already exists
        if response.status_code == 200 and len(response.json()) > 0:
            existing_role = response.json()[0]
            existing_role_name = existing_role.get("name")
            if existing_role_name == role_name:
                update_role_url = f"{roles_url}/{existing_role_name}"
                update_response = requests.put(update_role_url, data=json.dumps(role), headers=th, verify=False)
                update_response.raise_for_status()
                print(f"Role {role_name} updated successfully for client {client_uuid}.")
                continue
            else:
                raise Exception(f"Role name mismatch: expected {role_name}, found '{existing_role_name}'.")

        # Create a new role if it does not exist
        create_response = requests.post(roles_url, headers=th, data=json.dumps(role), verify=False)
        create_response.raise_for_status()
        print(f"Role {role_name} created successfully for client {client_uuid}.")


def get_client_roles(url, realm, token, client_uuid):
    roles_url = f"{url}/admin/realms/{realm}/clients/{client_uuid}/roles"
    response = requests.get(roles_url, headers=get_token_header(token), verify=False)
    response.raise_for_status()
    return response.json()


def get_client_roles_for_user(url, realm, token, user_id, client_uuid):
    roles_url = f"{url}/admin/realms/{realm}/users/{user_id}/role-mappings/clients/{client_uuid}"
    response = requests.get(roles_url, headers=get_token_header(token), verify=False)
    response.raise_for_status()
    return response.json()


def get_realm_roles(url, realm, token):
    roles_url = f"{url}/admin/realms/{realm}/roles"
    response = requests.get(roles_url, headers=get_token_header(token), verify=False)
    response.raise_for_status()
    return response.json()


def get_roles_for_user(url, realm, token, user_uuid):
    roles_url = f"{url}/admin/realms/{realm}/users/{user_uuid}/role-mappings/realm"
    response = requests.get(roles_url, headers=get_token_header(token), verify=False)
    response.raise_for_status()
    return response.json()


def get_client_uuid_by_client_id(url, realm, token, client_id):
    clients_url = f"{url}/admin/realms/{realm}/clients"
    response = requests.get(clients_url, headers=get_token_header(token), params={"clientId": client_id}, verify=False)
    response.raise_for_status()
    clients = response.json()
    if not clients:
        raise Exception(f"Client {client_id} not found in realm {realm}")
    return clients[0]["id"]


def get_client_role_by_role_name(url, realm, token, client_uuid, role_name):
    roles_url = f"{url}/admin/realms/{realm}/clients/{client_uuid}/roles"
    response = requests.get(roles_url, headers=get_token_header(token), params={"search": role_name}, verify=False)
    response.raise_for_status()
    roles = response.json()
    if not roles:
        raise Exception(f"Role {role_name} not found for client {client_uuid} in realm {realm}")
    return roles[0]



import subprocess

from griddot.provision_keycloak import get_realms
from tests.provision_keycloak_tests import wait_for_keycloak_and_get_token, build_keycloak_images, run_keycloak


def setup_keycloak_with_provisioner(clean, build_images):
    if build_images:
        build_keycloak_images()

    if clean:
        run_keycloak()

    cwd = "../../containers/keycloak-provisioner"
    subprocess.run(f"podman kube play --replace provisioner.yaml", check=True, shell=True, cwd=cwd)


def test_provisioner():
    setup_keycloak_with_provisioner(True, False)
    url, token = wait_for_keycloak_and_get_token()

    realms = get_realms(url, token)
    realms_names = [realm["realm"] for realm in realms]
    assert "master" in realms_names, "Master realm not found"
    assert "platform" in realms_names, "Platform realm not found"



import subprocess

def get_container_command():
    """
    Check if Podman is installed, if not, use Docker.
    Returns the command to use for container operations.
    """
    container_cmd = "podman"
    try:
        subprocess.run("podman --version", shell=True, check=True, capture_output=True)
    except subprocess.CalledProcessError:
        container_cmd = "docker"
    return container_cmd
